#!/usr/bin/python
"""Script for parsing a version 1 attestation file."""
import argparse
import binascii
import gzip
import struct

# Attestation structure is big endian.
# The header is composed of the handle, attribute count, and object size.
HEADER = '>III'
# The TLV is composed of the object type and its size.
TLV = '>II'
# Asymmetric key attestation objects start after 984 bytes.
CAVIUM_ATTESTATION_ASYM_OFFSET = 984
# Symmetric key attestation objects start after 24 bytes.
CAVIUM_ATTESTATION_SYM_OFFSET = 24


def get_contents_decompress_if_needed(attestation_file):
  try:
    with gzip.open(attestation_file, 'rb') as f:
      return f.read()
  except IOError:
    with open(attestation_file, 'rb') as f:
      return f.read()


def parse(attestation):
  _, attr_count, obj_size = struct.unpack_from(HEADER, attestation, 0)
  obj_header_size = struct.calcsize(HEADER)
  attestation = attestation[obj_header_size:]

  attributes = {}
  while attr_count > 0:
    # Parse each Attribute.
    attr_type, attr_len = struct.unpack_from(TLV, attestation, 0)
    attestation = attestation[struct.calcsize(TLV):]
    attributes['0x%04x' % attr_type] = binascii.hexlify(attestation[:attr_len])
    attr_count -= 1
    attestation = attestation[attr_len:]
  return attributes, obj_header_size + obj_size


def print_attributes(attributes):
  for attr in sorted(attributes):
    print('%s: %s' % (attr, attributes[attr]))


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument(
      'key_type', choices=['symmetric', 'asymmetric'], help='The key type.')
  parser.add_argument('attestation_file', help=' The name of attestation file.')
  args = parser.parse_args()

  attestation = get_contents_decompress_if_needed(args.attestation_file)
  if args.key_type == 'symmetric':
    print('Symmetric Key Attestation')
    attributes, offset = parse(attestation[CAVIUM_ATTESTATION_SYM_OFFSET:])
    print_attributes(attributes)
  else:
    print('Public Key Attestation')
    public_key_attributes, offset = parse(
        attestation[CAVIUM_ATTESTATION_ASYM_OFFSET:])
    print_attributes(public_key_attributes)
    print('Private Key Attestation')
    private_key_attributes, _ = parse(
        attestation[(CAVIUM_ATTESTATION_ASYM_OFFSET + offset):])
    print_attributes(private_key_attributes)


if __name__ == '__main__':
  main()

